

#
#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # V2.0
  # Name :        K2_ToolsBiscuitJoint  ###
  # Date :	      Aug 2011
  # Description : Removed CL Points; Fixed Spacing < 2
  
  
  

  # Author :       Keith Krueger & Sam Mitchel (with a big help from plugin forum:)
  #
 
require 'sketchup.rb'
require  'K2WS_Tools/KK_Input_parsing.rb'

class K2_ToolsBiscuitJoint  
   include KK_InputParsing
    
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@toolType = "Joint_Tool"
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsDowelHelp.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			reset
			self.dialog
			self.no_of_Offset_Input
			if @noOffsets >= 1
				self.input_Offsets
			end
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				reset
				self.dialog
				self.no_of_Offset_Input
				if @noOffsets >= 1
					input_Offsets
				end
			else
				reset
				self.readExistingData
			end
		end
		
	end
    
	
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false
	end
	
	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close	
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		
		@biscuitSize=@ans[0].to_s.strip;
		
		@spacing=@ans[1].to_s.strip;
		
		@quanity=KK_InputParsing.string_to_integer_formula(@ans[2].to_s.strip)
		if !@quanity then @quanity=@ans[2].to_i end
		
		@edge2CL=KK_InputParsing.string_to_length_formula(@ans[3].to_s.strip)
		if !@edge2CL then @edge2CL=@ans[3].to_l end
		
		@end2CL=KK_InputParsing.string_to_length_formula(@ans[4].to_s.strip)
		if !@end2CL then @end2CL=@ans[4].to_l end
		
		self.no_of_Offset_Input
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(@ans[4+i].to_s)
			if !@cl_offset[i-1] then @cl_offset[i-1]=@ans[4+i] end;
             
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0	
	end #readExistingData
	
	def dialog
		# open input data file in read mode	       
		self.read_data
		prompts = ["Select Biscuit Size ","Spacing","Quanity","Edge to CL","End to First CL"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip,@ans[4].to_s.strip]
		choices = ["#20|#10|#0|FF","Symetrical|Equal|User|Fixed","","",""] 
		results = inputbox prompts, defaults, choices, "Input Data for Biscuits "			
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@biscuitSize=results[0].to_s.strip; @ans[0] = results[0]
		
		@spacing=results[1].to_s.strip; @ans[1] = results[1]
		
		@quanity=KK_InputParsing.string_to_integer_formula(results[2].to_s.strip)
		if !@quanity then @quanity=results[2].to_i end; @ans[2] = results[2] 
		
		@edge2CL=KK_InputParsing.string_to_length_formula(results[3].to_s.strip)
		if !@edge2CL then @edge2CL=results[3].to_l end; @ans[3] = results[3]
                
		@end2CL=KK_InputParsing.string_to_length_formula(results[4].to_s.strip)
		if !@end2CL then @end2CL=results[4].to_l end;  @ans[4] = results[4]
 	
		# open input data file in write mode	       
		self.write_data
	end #dialog
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end

	def no_of_Offset_Input
		case @spacing
			when "Symetrical"
			if @quanity <= 3
				@noOffsets=0
			else   
				@noOffsets=(@quanity - 2)/2
			end #if @quanity <= 3      
			when "Equal"
				@noOffsets=0
			when "User"
				@noOffsets=@quanity - 1
			when "Fixed"
				if @quanity <= 1
					@noOffsets=0
				else   
					@noOffsets=1
				end
		end #case @spacing
	end #no_of_offsets
      
	def round_3places(n)  
		n=(n*1000).to_i
		n=n.to_f/1000      
	end #def round_3places
  
	def input_Offsets
		# open a file in read mode	       
		self.read_data
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			#@cl_offset[i-1] = ans[4+i]
			prompts = ["CL Distance Biscuit " + i.to_s + " to Biscuit " + i2.to_s + " " ]
     
			defaults= [@ans[4+i].to_s.strip]
                       
			results = inputbox prompts, defaults, "Input CL Distance between Biscuits"

			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(results[0])
			if !@cl_offset[i-1] then @cl_offset[i-1]=results[0] end; @ans[4+i] = results[0]
             
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0
		# Write an array of Input Data to the file
		self.write_data      
	end #def input_Offsets 

	def cal_xCL_points(x1,e_len,dir_x)
		@xcl=Array.new @quanity.to_i
		case @spacing
			when "Symetrical"
				#Debug UI.messagebox("Spacing = " + @spacing.to_s)
				if @quanity == 1
					@xcl[0]=x1+(@end2CL*dir_x)
					#Debug UI.messagebox("@xcl[0] = " + @xcl[0].to_s)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity == 2
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)        
				elsif @quanity == 3
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len/2)*dir_x)
					@xcl[2]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity >= 4
					#Debug UI.messagebox("Started Quanity = " + @quanity.to_s)
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
					@xcl[@quanity/2]=x1+((e_len/2)*dir_x)
					n =(@quanity-2)/2  #loop number
					sn = 1 #Starting number
					i = 0
					while n > 0
						temp_offset=@cl_offset[i]
						#Debug  UI.messagebox("CL Offset = "+ temp_offset.to_s)
						@xcl[sn]=@xcl[sn-1]+((temp_offset)*dir_x)
						@xcl[@quanity-1-sn]=@xcl[@quanity-sn]-((temp_offset)*dir_x)
						sn = sn + 1
							n = n -1
						i = i + 1
					end # while n > 0                
				end #if @quanity == 
				#Debug  @xcl.each {|e| UI.messagebox("Symetrical CL point = "+ e.to_s)}           
			when "Equal"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
				@xcl[0]=x1+(@end2CL*dir_x)
				@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
				n =(@quanity-2)  #loop number
				sn = 1 #Starting number
				while n > 0
					@xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
					sn = sn + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
			when "User"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				@xcl[0]=x1+(@end2CL*dir_x)
				n =(@quanity-1)  #loop number
				sn = 1 #Starting number
				i = 0
				while n > 0
					user_offset=@cl_offset[i]
					#user_offset=user_offset.to_l
					#Debug UI.messagebox("user_offset = "+ user_offset.to_s)
					@xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
					sn = sn + 1
					i = i + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
		when "Fixed"
			#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
			@xcl[0]=x1+(@end2CL*dir_x)
			n =(@quanity-1)  #loop number
			sn = 1 #Starting number
			while n > 0
				fixed_offset=@cl_offset[0]
				@xcl[sn]=@xcl[sn-1]+((fixed_offset)*dir_x)
				sn = sn + 1
				n = n -1
			end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Fixed CL point = "+ e.to_s)}         
		end #case @spacing
		#Debug UI.messagebox("End Spacing = " + @spacing.to_s)
	end #cal_xCL_points
  
    def points_BiscuitSlot(x,y,z,dz_dir)
		@pt=Array.new 16
		w=(0.156/2).inch
		case @biscuitSize
			when "#20"
				dx1=1.313.inch;	dx2=0.922.inch;	dz2=0.276.inch;	dx3=0.475.inch;	dz3=0.449.inch;	dz4=0.508.inch
			when "#10"
				dx1=1.219.inch;	dx2=0.850.inch;	dz2=0.238.inch;	dx3=0.436.inch;	dz3=0.386.inch;	dz4=0.436.inch
			when "#0"
				dx1=1.094.inch;	dx2=0.754.inch;	dz2=0.184.inch;	dx3=0.384.inch;	dz3=0.297.inch;	dz4=0.335.inch
			when "FF"
				dx1=0.656.inch;	dx2=0.461.inch;	dz2=0.138.inch;	dx3=0.238.inch;	dz3=0.224.inch;	dz4=0.254.inch
		end #case @biscuitSize
		dz2 = dz2 * dz_dir
		dz3 = dz3 * dz_dir
		dz4 = dz4 * dz_dir
		#Top Arc     
		@pt[0]=Geom::Point3d.new(x-dx1,y+w,z)
		@pt[1]=Geom::Point3d.new(x-dx2,y+w,z-dz2)
		@pt[2]=Geom::Point3d.new(x-dx3,y+w,z-dz3)
		@pt[3]=Geom::Point3d.new(x,y+w,z-dz4)
		@pt[4]=Geom::Point3d.new(x+dx3,y+w,z-dz3)
		@pt[5]=Geom::Point3d.new(x+dx2,y+w,z-dz2)
		@pt[6]=Geom::Point3d.new(x+dx1,y+w,z)
		#Bottom Arc
		@pt[7]=Geom::Point3d.new(x-dx1,y-w,z)
		@pt[8]=Geom::Point3d.new(x-dx2,y-w,z-dz2)
		@pt[9]=Geom::Point3d.new(x-dx3,y-w,z-dz3)
		@pt[10]=Geom::Point3d.new(x,y-w,z-dz4)
		@pt[11]=Geom::Point3d.new(x+dx3,y-w,z-dz3)
		@pt[12]=Geom::Point3d.new(x+dx2,y-w,z-dz2)
		@pt[13]=Geom::Point3d.new(x+dx1,y-w,z)
		#Center point for Tringular Faces top 
		@pt[14]=Geom::Point3d.new(x,y+w,z)
		#Center point for Tringular Faces bottom
		@pt[15]=Geom::Point3d.new(x,y-w,z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_BiscuitSlot
	
	def points_Biscuit(x,y,z)
		@ptbb=Array.new 14
		@ptbt=Array.new 14
		w=(0.156/2).inch
		case @biscuitSize
			when "#20"
				dx1b=1.110.inch; dz1b=0.113; dx2b=0.756.inch; dz2b=0.302.inch; dx3b=0.383.inch;	dz3b=0.417.inch; dz4b=0.456.inch
			when "#10"
				dx1b=1.051.inch; dz1b=0.075; dx2b=0.716.inch; dz2b=0.231.inch; dx3b=0.363.inch;	dz3b=0.326.inch; dz4b=0.358.inch
			when "#0"
				dx1b=0.932.inch; dz1b=0.074; dx2b=0.635.inch; dz2b=0.197.inch; dx3b=0.322.inch;	dz3b=0.272.inch; dz4b=0.297.inch
			when "FF"
				dx1b=0.562.inch; dz1b=0.030; dx2b=0.446.inch; dz2b=0.111.inch; dx3b=0.230.inch;	dz3b=0.195.inch; dz4b=0.224.inch
		end #case @biscuitSize
		
		#Top Face     
		@ptbt[0]=Geom::Point3d.new(x-dx1b,y+w,z-dz1b)
		@ptbt[1]=Geom::Point3d.new(x-dx2b,y+w,z-dz2b)
		@ptbt[2]=Geom::Point3d.new(x-dx3b,y+w,z-dz3b)
		@ptbt[3]=Geom::Point3d.new(x,y+w,z-dz4b)
		@ptbt[4]=Geom::Point3d.new(x+dx3b,y+w,z-dz3b)
		@ptbt[5]=Geom::Point3d.new(x+dx2b,y+w,z-dz2b)
		@ptbt[6]=Geom::Point3d.new(x+dx1b,y+w,z-dz1b)
		@ptbt[7]=Geom::Point3d.new(x+dx1b,y+w,z+dz1b)
		@ptbt[8]=Geom::Point3d.new(x+dx2b,y+w,z+dz2b)
		@ptbt[9]=Geom::Point3d.new(x+dx3b,y+w,z+dz3b)
		@ptbt[10]=Geom::Point3d.new(x,y+w,z+dz4b)
		@ptbt[11]=Geom::Point3d.new(x-dx3b,y+w,z+dz3b)
		@ptbt[12]=Geom::Point3d.new(x-dx2b,y+w,z+dz2b)
		@ptbt[13]=Geom::Point3d.new(x-dx1b,y+w,z+dz1b)
		#Bottom Face
		@ptbb[0]=Geom::Point3d.new(x-dx1b,y-w,z-dz1b)
		@ptbb[1]=Geom::Point3d.new(x-dx2b,y-w,z-dz2b)
		@ptbb[2]=Geom::Point3d.new(x-dx3b,y-w,z-dz3b)
		@ptbb[3]=Geom::Point3d.new(x,y-w,z-dz4b)
		@ptbb[4]=Geom::Point3d.new(x+dx3b,y-w,z-dz3b)
		@ptbb[5]=Geom::Point3d.new(x+dx2b,y-w,z-dz2b)
		@ptbb[6]=Geom::Point3d.new(x+dx1b,y-w,z-dz1b)
		@ptbb[7]=Geom::Point3d.new(x+dx1b,y-w,z+dz1b)
		@ptbb[8]=Geom::Point3d.new(x+dx2b,y-w,z+dz2b)
		@ptbb[9]=Geom::Point3d.new(x+dx3b,y-w,z+dz3b)
		@ptbb[10]=Geom::Point3d.new(x,y-w,z+dz4b)
		@ptbb[11]=Geom::Point3d.new(x-dx3b,y-w,z+dz3b)
		@ptbb[12]=Geom::Point3d.new(x-dx2b,y-w,z+dz2b)
		@ptbb[13]=Geom::Point3d.new(x-dx1b,y-w,z+dz1b)   
		#Debug @ptbt.each {|e| UI.messagebox("point biscuit top = "+ e.to_s)}
		#Debug @ptbb.each {|e| UI.messagebox("point biscuit bot = "+ e.to_s)}    
	end #points_Biscuit
  
	def points_BiscuitCL(x,y)
		@clpt=Array.new 4
		cline=0.03.inch
		@clpt[0]=Geom::Point3d.new(x+(cline),y,0)
		@clpt[1]=Geom::Point3d.new(x,y,0)
		@clpt[2]=Geom::Point3d.new(x,y+(cline),0)
		@clpt[3]=Geom::Point3d.new(x,y,0)
		#Debug @clpt.each {|e| UI.messagebox("CL point Biscuit = "+ e.to_s)} 
	end #def points_BiscuitCL
	
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select 1st Component for Biscuit Joint:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select First Edge in component 1 Parallel to Biscuit Length:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities;
				@status=1; if @toolType == "SingleComp_Tool" then @status=2 end
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end # if@comp1			
		elsif @status==1	#select 2nd component
			ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
			if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp2); @ent2=@comp2.definition.entities;  @status=2
				@tform2 = @comp2.transformation.inverse
				onMouseMove(flags, x, y, view)
				@sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
			else
				UI.messagebox "Selection invalid, must be component"
			end #@entity			
		elsif @status==2 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@tform = @ip.transformation
					@edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position
					@status = 3
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid Use Right Click & Hide Interfering Comp"
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid "	
			end #if(@ip.valid?)
		elsif @status==3 #select short edge, thickness
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position 
					if @toolType != "SingleComp_Tool" then @comp2.hidden=false end
					if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
						orig=@e1sp
						p1=@e1sp
						p2=@e1ep
					end #if ((@e1sp==
					if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
						orig=@e1ep
						p2=@e1sp
						p1=@e1ep
					end  #if ((@e1ep==
					if orig==@e2sp
						p3=@e2ep
						else
						p3=@e2sp
					end #if orig              
					#find face that has edges 1 and 2
					face=@edge1.common_face @edge2
					if (face==nil)#14                    #get valid face
						@oops=4
					else 
						#Debug UI.messagebox"got valid face"
						#get points in local coord system for face
						#xaxis parallel to @edge1
						x,y,z=face.normal.axes
						#get vector parallel to @edge1              			         			          
						ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
						xx=ar.normalize
						yy=z.cross xx
						localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
						backt=Geom::Transformation.axes(orig,xx,yy,z)
						edges=face.outer_loop.edges
						eind=edges.index(@edge1)
						e_len=@edge1.length
						#Points on face in local coordinates
						p1=p1.transform(localt)
						p2=p2.transform(localt)
						p3=p3.transform(localt)           
						x1=p1.x
						x2=p2.x
						y1=p1.y
						y2=p3.y
						if x1>x2 then dir_x = -1 end
						if x1<x2 then dir_x = +1 end
						if y1<y2 then dir_y = +1 end
						if y1>y2 then dir_y = -1 end						                                                                       
						cal_xCL_points(x1,e_len,dir_x)  
						#Debug UI.messagebox"Returned from calculating the xCL points"
						#Debug @xcl.each {|element| UI.messagebox("Cal @xcl Points = "+ element.to_s)}
										
						#START LOOP
						i=0
						n=@quanity
						Sketchup.active_model.start_operation "Biscuit Slots"                
						while n > 0               			          
							ycl = y1+(@edge2CL*dir_y)
							#Debug UI.messagebox("yc1 = "+ ycl.to_s)
							#Calculate points for Biscuit Slot
							dz_dir = 1 # Component 1
							points_BiscuitSlot(@xcl[i],ycl,0,dz_dir)
							#Debug @pt.each {|element| UI.messagebox("@pt point comp 1 = "+ element.to_s)}               
							#Transform Biscuit Slot Points to Component 1 Coordinates
							@pt=@pt.collect {|point| point.transform(backt)} #Transform to Comp 1               
							#Debug @pt.each {|element| UI.messagebox("@pt point comp 1 = "+ element.to_s)}
							#Add Biscuit Slot
							if @toolType=="Joint_Tool" then @comp2.hidden=false  end
							@comp1.hidden=false
							@entity=@comp1
							codef=@entity.definition
							@sel.clear
							@sel.add(@comp1)
							# CREATING SLOT IN COMPONENT 1
							#Add Slot Face then Delete to create slot opening
							cface=codef.entities.add_face(@pt[0],@pt[6],@pt[13],@pt[7])
							codef.entities.erase_entities cface                
							#Loop for adding faces needed to create slot
							for k in 0..5
								k1=k+1
								k8=k+8
								k7=k+7                                                
								#Add back curved face of slot
								cface=codef.entities.add_face(@pt[k],@pt[k1],@pt[k8],@pt[k7])                                
								#Add  face to top of slot
								cface=codef.entities.add_face(@pt[0..6])
								#Add  face to bottom of slot
								cface=codef.entities.add_face(@pt[7..13])
							end #for k in 0..5                
							#Feature added to First component at this point						
							if @toolType=="Joint_Tool"
								@entity=@comp2
								codef=@entity.definition
								@sel.clear
								@sel.add(@comp2)
								@statusstring= "Adding Feature to Second Component"
								Sketchup::set_status_text @statusstring,SB_PROMPT
								#Calculate points for Biscuit Slot in comp 2
								dz_dir = -1 # Component 2
							    points_BiscuitSlot(@xcl[i],ycl,0,dz_dir)
								@pt=@pt.collect {|point| point.transform(backt)} 
								@ptg=@pt.collect {|point| point.transform(@tform)} #Transform to Global	                
								#Debug @ptg.each {|e| UI.messagebox("Global point c1g = "+ e.to_s)}                  
								@ptC2=@ptg.collect {|point| point.transform(@tform2)} #Transform to Comp 2
								# CREATING SLOT IN COMPONENT 2
								#Add Slot Face then Delete to create slot opening
								cface=codef.entities.add_face(@ptC2[0],@ptC2[6],@ptC2[13],@ptC2[7])
								codef.entities.erase_entities cface                
								#Loop for adding faces needed to create slot
								for k in 0..5
									k1=k+1
									k8=k+8
									k7=k+7                                                
									#Add back curved face of slot
									cface=codef.entities.add_face(@ptC2[k],@ptC2[k1],@ptC2[k8],@ptC2[k7])                                
									#Add  face to top of slot
									cface=codef.entities.add_face(@ptC2[0..6])
									#Add  face to bottom of slot
									cface=codef.entities.add_face(@ptC2[7..13])
								end #for i in 0..5                
								#Feature added to second component at this point
							end #if @toolType=="Joint_Tool"
							i=i+1
							n=n-1
							#Debug  UI.messagebox("Number Holes Left = " + n.to_s )
						end #while n > 0
						Sketchup.active_model.commit_operation
						#END LOOP
						result=UI.messagebox "Slots Complete; Add Biscuits to Slots? ", MB_YESNO              
						if result==6  #Add Biscuits to  slots
							Sketchup.active_model.start_operation "Biscuits"
							i=0
							n=@quanity
							while n > 0
								Sketchup.active_model.start_operation "Biscuits"
								@sel.clear
								model = Sketchup.active_model
								entities = model.active_entities
								group = entities.add_group
								entities = group.entities
								# Calculate New Points for Creating biscuits
								points_Biscuit(@xcl[i],ycl,0) #@ptbt[] @ptbb[]
								@ptbt=@ptbt.collect {|point| point.transform(backt)} #Translate to Comp 1 Coord
								@ptbt=@ptbt.collect {|point| point.transform(@tform)} #Translate to Global Coord 
								@ptbb=@ptbb.collect {|point| point.transform(backt)} #Translate to Comp 1 Coord
								@ptbb=@ptbb.collect {|point| point.transform(@tform)} #Translate to Global Coord 
								#Add Biscuit
								bface=entities.add_face(@ptbt)
								bface.reverse!
								bface=entities.add_face(@ptbb)
								
								for k in -1...13
									k2=k+1
									bface=entities.add_face(@ptbb[k],@ptbt[k],@ptbt[k2],@ptbb[k2])
								end #for k                		
								# Convert the Group to a Component 
								component_instance=group.to_component
								component_instance.definition.name="Part Biscuit_" + @biscuitSize
								i=i+1
								n=n-1	
							end #while n loop
							Sketchup.active_model.commit_operation
						end #result==6
						#Program is Complete and ends
						Sketchup.send_action "selectSelectionTool:"          
					end #face==nil                        
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid"	
			end #if(@ip.valid?)	
		end #elsif @status==
	end # def onLButtonDown

	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
	def onRButtonDown(flags,x,y,view)
		reset
		self.dialog
		self.no_of_Offset_Input
		if @noOffsets >= 1
			input_Offsets
		end
	end
	
end #end class K2_ToolsBiscuitJoint

class K2_ToolsBiscuitSlot  < K2_ToolsBiscuitJoint
    def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @toolType = "SingleComp_Tool"
	end
	
	def onMouseMove(flags, x, y, view)  
		Sketchup::set_status_text("Select Component for Biscuit Slots:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select First Edge in component Parallel to Biscuit Length:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end

end # class K2_ToolsBiscuitSlot

 file_loaded("K2_ToolsBiscuitJoint.rb")
 
