

#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#	Name:	K2_ToolsBlindDovetail1.rb
#  Usage:	1.Create Half Blind dovetail Pins on the Pin Board that is a Component.
#		 	2.Create dovetail Tails on the Component Tail Board to fit the Half Blind pins.
#  Notes:	Tails may need additional clean up. OR try selecting from other side of board.
#		    	 
#		 
# Author:	Keith Krueger and Sam Mitchel (Sam programed the geometry code)		 
#	Date:	Aug 2011
#------------------------------------------------------------------------------------------------

require 'sketchup'
require  'K2WS_Tools/KK_Input_parsing.rb'
class K2_BlindDovetailPins
	include KK_InputParsing
	
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@labelText = "Half Blind Dovetail PIN Dimensions"
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsBlindDT_Help.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			self.reset
			self.dialog
			self.no_of_Offset_Input
			if @noOffsets >= 1
				self.input_Offsets
			end
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				self.reset
				self.dialog
				self.no_of_Offset_Input
				if @noOffsets >= 1
					input_Offsets
				end
			else
				self.reset
				self.readExistingData
			end
		end
		
	end
	
	def reset
		@best = nil; @comp = nil; @sel.clear; @status = 0; @drawn = false
	end
	
	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close		
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		
		@spc=@ans[0].to_s.strip
		
		@ang=KK_InputParsing.string_to_float_formula(@ans[1].to_s.strip)
		if !@ang then @ang=@ans[1] end; 
		
		@hpt=KK_InputParsing.string_to_length_formula(@ans[2].to_s.strip)
		if !@hpt then @hpt=@ans[2] end; 
		
		@fpt=KK_InputParsing.string_to_length_formula(@ans[3].to_s.strip)
		if !@fpt then @fpt=@ans[3] end; 
		
		@plen=KK_InputParsing.string_to_length_formula(@ans[4].to_s.strip)
		if !@plen then @plen=@ans[4] end;
		
		@blPinWth=KK_InputParsing.string_to_length_formula(@ans[5].to_s.strip)
		if !@blPinWth then @blPinWth=@ans[5] end
		
		@spacing=@ans[6].to_s.strip;
		
		@quanity=KK_InputParsing.string_to_integer_formula(@ans[7].to_s.strip)		
		if !@quanity then @quanity=@ans[7].to_i end;
		
		if @spc == "Degree" then	@rise = Math.tan(@ang.degrees) end
		if @spc == "Slope" then	@rise = @ang end
		
		self.no_of_Offset_Input
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(@ans[7+i].to_s)
			if !@cl_offset[i-1] then @cl_offset[i-1]=@ans[7+i] end;
             
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0	
	end #readExistingData
	
	def dialog
		# open a file in read mode
		self.read_data
		#@spc = @ans[0].to_s.strip; @ang = @ans[1]; @hpt = @ans[2]; @fpt = @ans[3]; @plen = @ans[4]; @blPinWth = @ans[5] @spacing = @ans[6].to_s.strip; @quanity = @ans[7]
		prompts = ["Angle Units","Angle","Half Pin T @ Root ","FullPin T @ Root","Pin Length","Blind Pin Width","Spacing","Quanity"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip,@ans[4].to_s.strip,@ans[5].to_s.strip,@ans[6].to_s.strip,@ans[7].to_s.strip]
		choices = ["Degree|Slope","","","","","","Symetrical|Equal|User|Fixed",""]	 
		results = inputbox prompts, defaults, choices, @labelText
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@spc=results[0].to_s.strip; @ans[0] = results[0]
		
		@ang=KK_InputParsing.string_to_float_formula(results[1].to_s.strip)
		if !@ang then @ang=results[1] end; @ans[1] = results[1]
		
		@hpt=KK_InputParsing.string_to_length_formula(results[2].to_s.strip)
		if !@hpt then @hpt=results[2] end; @ans[2] = results[2]
		
		@fpt=KK_InputParsing.string_to_length_formula(results[3].to_s.strip)
		if !@fpt then @fpt=results[3] end; @ans[3] = results[3]
		
		@plen=KK_InputParsing.string_to_length_formula(results[4].to_s.strip)
		if !@plen then @plen=results[4] end; @ans[4] = results[4]
		
		@blPinWth=KK_InputParsing.string_to_length_formula(results[5].to_s.strip)
		if !@blPinWth then @blPinWth=results[5] end; @ans[5] = results[5]
		
		@spacing=results[6].to_s.strip; @ans[6] = results[6]
		
		@quanity=KK_InputParsing.string_to_integer_formula(results[7].to_s.strip)
		if !@quanity then @quanity=results[7].to_i end; @ans[7] = results[7]
		
		# Write an array of Input Data to the file
		self.write_data
		if @spc == "Degree" then	@rise = Math.tan(@ang.degrees) end
		if @spc == "Slope" then	@rise = @ang end
	end
	
	def no_of_Offset_Input
		case @spacing
			when "Symetrical"
				if @quanity <= 3
					@noOffsets=0
				else	 
					@noOffsets=(@quanity - 2)/2
				end #if @quanity <= 3		
			when "Equal"
				@noOffsets=0
			when "User"
				@noOffsets=@quanity - 1
			when "Fixed"
				@noOffsets=1
		end #case @spacing
	end #no_of_offsets
		
	def round_3places(n)	
		n=(n*1000).to_i
		n=n.to_f/1000		
	end #def round_3places
	
	def input_Offsets
		# open a file in read mode	       
		self.read_data
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			prompts = ["CL Distance Pin " + i.to_s + " to Pin " + i2.to_s + " " ]
     
			defaults= [@ans[8+i].to_s.strip]
                       
			results = inputbox prompts, defaults, "Input CL Distance between Pins"

			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
			if !@cl_offset[i-1] then @cl_offset[i-1]=results[0] end; @ans[8+i] = results[0]
             
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0
		# Write an array of Input Data to the file
		self.write_data      
	end #def input_Offsets 
	
	def cal_xCL_points(x1,e_len)
		@xcl=Array.new @quanity
		case @spacing
			when "Symetrical"
				#Debug UI.messagebox("Spacing = " + @spacing.to_s)
				if @quanity == 1
					@xcl[0]=x1+(@hpt-(@fpt/2))
					#Debug UI.messagebox("@xcl[0] = " + @xcl[0].to_s)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity == 2
					@xcl[0]=x1+(@hpt-(@fpt/2))
					@xcl[1]=x1+e_len-((@fpt/2)-@hpt)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)		
				elsif @quanity == 3
					@xcl[0]=x1+(@hpt-(@fpt/2))			
					@xcl[1]=x1+(e_len/2)
					@xcl[2]=x1+e_len-(@hpt-(@fpt/2))
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity >= 4
					#Calculate First, Last, Middle
					@xcl[0]=x1+(@hpt-(@fpt/2))
					@xcl[@quanity-1]=x1+e_len-(@hpt-(@fpt/2))
					@xcl[@quanity/2]=x1+(e_len/2)
					# Loop for the remaining CL
					n =(@quanity-2)/2	#loop number
					sn = 1 #Starting number
					i = 0
					while n > 0
						temp_offset=@cl_offset[i]
						@xcl[sn]=@xcl[sn-1]+(temp_offset)
						@xcl[@quanity-1-sn]=@xcl[@quanity-sn]-(temp_offset)
						sn = sn + 1
						n = n -1
						i = i + 1
					end # while n > 0				
				end #if @quanity ==
				#Debug @xcl.each {|e| UI.messagebox("Symetrical CL point = "+ e.to_s)}			
			when "Equal"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				equal_offset=(e_len-(@hpt-(@fpt/2)))/(@quanity-1)
				@xcl[0]=x1+(@hpt-(@fpt/2))
				@xcl[@quanity-1]=x1+(e_len-(@hpt-(@fpt/2)))
				n =(@quanity-2)	#loop number
				sn = 1 #Starting number
				while n > 0
					@xcl[sn]=@xcl[sn-1]+(equal_offset)
					sn = sn + 1
					n = n -1
				end # while n > 0
				#Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}		
			when "User"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				@xcl[0]=x1+(@hpt-(@fpt/2))
				n =(@quanity-1)	#loop number
				sn = 1 #Starting number
				i = 0
				while n > 0
					user_offset=@cl_offset[i]
					#user_offset=user_offset.to_l
					#Debug UI.messagebox("user_offset = "+ user_offset.to_s)
					@xcl[sn]=@xcl[sn-1]+(user_offset)
					sn = sn + 1
					i = i + 1
					n = n -1
				end # while n > 0
				#Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
			when "Fixed"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				@xcl[0]=x1+(@hpt-(@fpt/2))
				n =(@quanity-1)	#loop number
				sn = 1 #Starting number
				while n > 0
					fixed_offset=@cl_offset[0]
					@xcl[sn]=@xcl[sn-1]+fixed_offset
					sn = sn + 1
					n = n -1
				end # while n > 0
				#Debug @xcl.each {|e| UI.messagebox("Fixed CL point = "+ e.to_s)}		 
		end #case @spacing
	end #cal_xCL_points
		

	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("SELECT Dovetail PIN Board Component:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("SELECT OUTSIDE WIDTH EDGE on End of PIN Board Component:   [SPACE BAR] to Exit",SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select TOP THICKNESS EDGE on End of PIN Board Component:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip	 
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities; @status=1
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end #@comp1			
		elsif @status==1 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge1 = @ip.edge
					@status = 2
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox"Other Component is Interfering with pick: Use Right Button Click and Hide before Continuing!"
					self.reset
				end
			end
		elsif @status==2 #select short edge, thickness 
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; 
					@mod.start_operation "Dovetail Pins"
					wid = @edge1.length; @pwid = @edge2.length
					#p2 = @ip.position; p2.transform! @tr2; edge=@ip.edge; pt,x2=edge.line
					#(pt = edge.end.position; x2.reverse!) if pt.distance(p2) > @ip.edge.length/2
					
					#Calculate Change in x due to @rise across thickness of Pin Board
					del_x = @rise * @blPinWth
					xcl = 1.0.to_l	#for testing dimension from outside edge to cl of next pin
					
					p0 = Geom.intersect_line_line(@edge1.line,@edge2.line)
					p1,x1=@edge1.line; x1.reverse! if p0 != p1
					p1,ya=@edge2.line; ya.reverse! if p0 != p1
					pa=p0.offset(ya,@pwid)
			        p0=p0.offset(ya,@pwid-@blPinWth) #half blind Dovetail Pins
					#Calculate offset points
					#no_of_Offset_Input
					#Get CL points for the pin features
					p0x = 0.0 #p0.x <--------------- changed this from p0.x to 0.0
					cal_xCL_points(p0x,wid)				
					
					#Debug UI.messagebox("del_x = " + del_x.to_s)
					
					i = 0
					for i in 0...@quanity-1	
						pp1=p0.offset(x1,@xcl[i]+@fpt/2) #pp1 (Pin Point 1) starts at half pin thickness from first @xcl
						pp2=pa.offset(x1,+@xcl[i]+@fpt/2+del_x)
						pp3=pa.offset(x1,@xcl[i+1]-@fpt/2-del_x)
						pp4=p0.offset(x1,@xcl[i+1]-@fpt/2)
												
							face= @ent1.add_face(pp1,pp2,pp3,pp4)
							face.pushpull -@plen
						
						 
					end
					@mod.commit_operation
					self.reset
					#Sketchup.send_action "selectSelectionTool:"
				else
					UI.beep
				end
			end
		end
	end#def onLButtonDown
	
	def onRButtonDown(flags,x,y,view)
		self.reset
		self.dialog
		self.no_of_Offset_Input
		if @noOffsets >= 1
			input_Offsets
		end
	end
		
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end

end #of class K2_BlindDovetailPins

class K2_BlindDovetailTails 	< K2_BlindDovetailPins

	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@labelText = "Tails to FIT these Half Blind PIN Dimensions"
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL 'K2_ToolsBlindDT_Help.jpg'
			UI.messagebox"created file"
			self.reset
			self.dialog
			self.no_of_Offset_Input
			if @noOffsets >= 1
				self.input_Offsets
			end
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				self.reset
				self.dialog
				self.no_of_Offset_Input
				if @noOffsets >= 1
					input_Offsets
				end
			else
				self.reset
				self.readExistingData
			end
		end
	end #initialize

	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("SELECT Dovetail TAIL Board Component:   RIGHT CLICK to Change Data:   SPACE BAR to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select Outside WIDTH EDGE on End of Tail Board Component:   SPACE BAR to Exit",SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select TOP LENGTH EDGE on Outside of Tail Board Component:   SPACE BAR to Exit",SB_PROMPT) if @status==2
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip	 
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities; @status=1
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end #@comp1			
		elsif @status==1 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge1 = @ip.edge
					@status = 2
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox"Other Component is Interfering with pick: Use Right Button Click and Hide before Continuing!"
					self.reset
				end
			end
		elsif @status==2 #select long edge, length 
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; 
					@mod.start_operation "Box Joint"
					wid = @edge1.length;
					#Tail Length Input Data
					read_data 
					prompts = ["Tail Length"]
                    defaults= [@ans[8].to_s.strip]                  
                    results = inputbox prompts, defaults, "Dovetail Tails Length"
					if !results then;Sketchup.send_action "selectSelectionTool:";return;end
					
					@pwid=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
		            if !@pwid then @pwid=results[0] end; @ans[8] = results[0]
					
					write_data # Save Tail Length Data
					
					#Calculate Change in x due to @rise across thickness of Pin Board
					del_x = @rise * @pwid
					xcl = 1.0.to_l	#for testing dimension from outside edge to cl of next pin
					#Calculate Starting Points
					p0 = Geom.intersect_line_line(@edge1.line,@edge2.line)
					p1,x1=@edge1.line; x1.reverse! if p0 != p1
					p1,ya=@edge2.line; ya.reverse! if p0 != p1
					#UI.messagebox("ya = "+ ya.to_s)
					#UI.messagebox("hgt = "+ hgt.to_s)
					pa=p0.offset(ya,@pwid)
					
					#Get CL points for the pin features
					p0x = 0.0 #p0.x <--------------- changed this from p0.x to 0.0
					cal_xCL_points(p0x,wid)					

					pp1=pa; pp2=p0
					for i in 0...@quanity-1
						pp3=p0.offset(x1,@xcl[i]+@fpt/2)
						pp4=pa.offset(x1,@xcl[i]+@fpt/2+del_x)
						face= @ent1.add_face(pp1,pp2,pp3,pp4)
						face.pushpull -@plen
						pp1=pa.offset(x1,@xcl[i+1]-@fpt/2-del_x)
						pp2=p0.offset(x1,@xcl[i+1]-@fpt/2) #pp1 (Pin Point 1) starts at half pin thickness from first @xcl
					end
					pp3=p0.offset(x1,wid); pp4=pa.offset(x1,wid)
					face= @ent1.add_face(pp1,pp2,pp3,pp4)
					face.pushpull -@plen
					
					@mod.commit_operation
					self.reset
					#Sketchup.send_action "selectSelectionTool:"
				else
					UI.beep
				end
			end
		end
	end#def onLButtonDown

end #class K2_BlindDovetailTails


#-----------------------------------------------------------------------------
file_loaded("K2_ToolsBlindDovetail1.rb") 
