
#

#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#  Name:	K2_Tools_BoxJoint.rb
# Usage:	Create Fingers/Spaces on end of 1 Component.  (Joint did not work consistantly)
#         
# Notes:	Removed Joint Tool not reliable 
#           Draw each set of fingers individually
#
#  Date:	Aug 2011
#------------------------------------------------------------------------------------------------

require 'sketchup'
require  'K2WS_Tools/KK_Input_parsing.rb'

class K2_BoxFingers
	
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsBoxJointHelp.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			self.reset
			self.dialog
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				self.reset
				self.dialog
			else
				self.reset
				self.readExistingData
			end
		end
		
	end
	
	def reset
		@best = nil; @comp = nil; @sel.clear; @status = 0; @drawn = false
		
	end
	
	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close		
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		@opt =  @ans[0].to_s.strip == "Space"
		@typ=@ans[0].to_s.strip;  @opt =  @typ == "Space"
		
        @spc=KK_InputParsing.string_to_length_formula(@ans[1].to_s.strip)
		if !@spc then @spc=@ans[1] end;
		
        @fng=KK_InputParsing.string_to_length_formula(@ans[2].to_s.strip)
		if !@fng then @fng=@ans[2] end;
        
		@qty=KK_InputParsing.string_to_integer_formula(@ans[3].to_s.strip)
		if !@qty then @qty=@ans[3].to_i end; 
			
	end #readExistingData

	def dialog
		# open a file in read mode	       
		self.read_data
		prompts = [" Start With:","       Thickness:","      Length:","     Quanity:"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip]
		choices = ["Space|Finger","","",""]  
		results = inputbox prompts, defaults, choices, "Input Data Box Joint Fingers/Spaces"
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@opt =  @ans[0].to_s.strip == "Space"
		@typ=results[0].to_s.strip; @ans[0] = results[0]; @opt =  @typ == "Space"
		
        @spc=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
		if !@spc then @spc=results[1] end; @ans[1] = results[1]
		
        @fng=KK_InputParsing.string_to_length_formula(results[2].to_s.strip)
		if !@fng then @fng=results[2] end; @ans[2] = results[2]
        
		@qty=KK_InputParsing.string_to_integer_formula(results[3].to_s.strip)
		if !@qty then @qty=results[3].to_i end; @ans[3] = results[3]
		
		# Write an array of Input Data to the file
		self.write_data
	end

	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select Component to get Box Joint Fingers:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select Outside Width Edge on Component:   [SPACE BAR] to Exit",SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select Top Thickness Edge on Component:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Width,Length:", SB_VCB_LABEL)
		Sketchup::set_status_text("#{@spc},#{@fng}", SB_VCB_VALUE)
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end


  	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities; @status=1
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end #@comp1			
		elsif @status==1 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge1 = @ip.edge
					@status = 2
					onMouseMove(flags, x, y, view)
				else
				  UI.messagebox"Other Component is Interfering with pick: Use Right Button Click and Hide before Continuing!"
				  self.reset
				end
			end
		elsif @status==2 #select short edge, thickness 
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; 
					@mod.start_operation "Box Joint"
					wid = @edge1.length; hgt = @edge2.length; @fng = hgt if @fng==0
					#p2 = @ip.position; p2.transform! @tr2; edge=@ip.edge; pt,x2=edge.line
					#(pt = edge.end.position; x2.reverse!) if pt.distance(p2) > @ip.edge.length/2
					p0 = Geom.intersect_line_line(@edge1.line,@edge2.line)
					p1,x1=@edge1.line; x1.reverse! if p0 != p1
					p1,ya=@edge2.line; ya.reverse! if p0 != p1
					for i in 0...@qty  #(wid/@spc).floor
						p1a=p0.offset(ya,hgt); #p1b=pt.offset(ya,hgt)
						p2a=p1a.offset(x1,@spc); #p2b=p1b.offset(x2,@spc)
						p3a=p0.offset(x1,@spc); #p3b=pt.offset(x2,@spc)
						if @opt
							face= @ent1.add_face(p0,p1a,p2a,p3a)
							face.pushpull -@fng
						end
						p0=p3a;  @opt = !@opt
					end
					@mod.commit_operation
					self.reset
				else
					UI.beep
				end
			end
		end
	end #def onLButtonDown
	
	def onRButtonDown(flags,x,y,view)
		self.reset
		self.dialog
	end
		
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
end # of class K2_BoxFingers





#-----------------------------------------------------------------------------
file_loaded("KK_BoxJointTool.rb") 
