


#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # V2.0
  # Name :        K2_ToolsDominoJoint.rb ###
  # Date :	      August 2011
  # Description : Removed CL Points; Fixed Spacing < 2
  #
  #
  # Author :       Keith Krueger  (with a big help from plugin forum:)
  #
 
require 'sketchup.rb'
require  'K2WS_Tools/KK_Input_parsing.rb'
 
class K2_Tools_DominoJoint 
   include KK_InputParsing
    
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@inpt = Sketchup::InputPoint.new           
		@tinpt= Sketchup::InputPoint.new
		@toolType = "Joint_Tool"
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsDowelHelp.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			reset
			self.dialog
			self.no_of_Offset_Input
			if @noOffsets >= 1
				self.input_Offsets
			end
			self.mortiseDepth
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				reset
				self.dialog
				self.no_of_Offset_Input
				if @noOffsets >= 1
					input_Offsets
				end
				self.mortiseDepth
			else
				reset
				self.readExistingData
			end
		end
		
	end
	
	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close		
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		
		@dominoSize=@ans[0].to_s.strip; 
		
		@spacing=@ans[1].to_s.strip; 
                                
		@widthSetting=@ans[2].to_s.strip; 
                
		@quanity=KK_InputParsing.string_to_integer_formula(@ans[3].to_s.strip)
		if !@quanity then @quanity=@ans[3].to_i end; 
                
		@edge2CL=KK_InputParsing.string_to_length_formula(@ans[4].to_s.strip)
		if !@edge2CL then @edge2CL=@ans[4].to_l end; 
                
		@end2CL=KK_InputParsing.string_to_length_formula(@ans[5].to_s.strip)
		if !@end2CL then @end2CL=@ans[5].to_l end; 
		
		@mdC1=KK_InputParsing.string_to_length_formula(@ans[6].to_s.strip)
		if !@mdC1 then @mdC1=@ans[6].to_l end; 
       
		@mdC2=KK_InputParsing.string_to_length_formula(@ans[7].to_s.strip)
		if !@mdC2 then @mdC2=@ans[7].to_l end; 
		self.no_of_Offset_Input
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(@ans[7+i].to_s)
			if !@cl_offset[i-1] then @cl_offset[i-1]=@ans[7+i] end;
             
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0	
	end #readExistingData
	
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false
	end

	def dialog
		# open input data file in read mode	       
		self.read_data
		prompts = ["Select Domino Size ","Spacing","Width Setting","Quanity","Edge to CL","End to First CL"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip,@ans[4].to_s.strip,@ans[5].to_s.strip]
		choices = ["10x24x50|5x19x30|8x22x40|8x22x50|6x20x40","Symetrical|Equal|User|Fixed","Standard|Middle|Wide","","",""]  
		results = inputbox prompts, defaults, choices, "Input Data for Domino Joint "		
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@dominoSize=results[0].to_s.strip; @ans[0] = results[0]
		
		@spacing=results[1].to_s.strip; @ans[1] = results[1]
                                
		@widthSetting=results[2].to_s.strip; @ans[2] = results[2]
                
		@quanity=KK_InputParsing.string_to_integer_formula(results[3].to_s.strip)
		if !@quanity then @quanity=results[3].to_i end; @ans[3] = results[3]
                
		@edge2CL=KK_InputParsing.string_to_length_formula(results[4].to_s.strip)
		if !@edge2CL then @edge2CL=results[4].to_l end; @ans[4] = results[4]
                
		@end2CL=KK_InputParsing.string_to_length_formula(results[5].to_s.strip)
		if !@end2CL then @end2CL=results[5].to_l end;  @ans[5] = results[5] 
		
		# Write an array of Input Data to the file
		self.write_data
	end #dialog
	
	

	def mortiseDepth
		# open input data file in read mode	       
		self.read_data	
		prompts = ["Mortise Depth Comp 1","Mortise Depth Comp 2"]
		defaults= [@ans[6].to_s.strip,@ans[7].to_s.strip]
		results = inputbox prompts, defaults, "Input Data for Domino Mortises"
		
   		@mdC1=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
		if !@mdC1 then @mdC1=results[0].to_l end; @ans[6] = results[0]
       
		@mdC2=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
		if !@mdC2 then @mdC2=results[1].to_l end; @ans[7] = results[1]
		# open a file in write mode	       
		self.write_data
  
	end #define mortiseDepth
  
	def dominoDimensions
		case @dominoSize
		when "10x24x50"
			@tt=10.mm; @wt=24.mm; @lt=50.mm; 
		when "5x19x30"
			@tt=5.mm; @wt=19.mm; @lt=30.mm; 
		when "8x22x40"
			@tt=8.mm; @wt=22.mm; @lt=40.mm; 
		when "8x22x50"
			@tt=8.mm; @wt=22.mm; @lt=50.mm; 
		when "6x20x40"
			@tt=6.mm; @wt=20.mm; @lt=40.mm; 
		end # case @dominoSize
      
    case @widthSetting
		when "Standard"
			@wm=@wt; @wf=@wm-@tt; @xf=@wf/2; 
		when "Middle"
			@wm=@wt+6.mm; @wf=@wm-@tt; @xf=@wf/2
		when "Wide"
			@wm=@wt+10.mm; @wf=@wm-@tt; @xf=@wf/2        
		end # case @widthSetting
		@mdC1=round_3places(@mdC1)
		@mdC2=round_3places(@mdC2)
  end #dominoDimensions
  
	def no_of_Offset_Input
		case @spacing
			when "Symetrical"
				if @quanity <= 3
					@noOffsets=0
				else   
					@noOffsets=(@quanity - 2)/2
				end #if @quanity <= 3      
			when "Equal"
				@noOffsets=0
			when "User"
				@noOffsets=@quanity - 1
			when "Fixed"
				if @quanity <= 1
					@noOffsets=0
				else   
					@noOffsets=1
				end
		end #case @spacing
	end #no_of_offsets
      
	def round_3places(n)  
		n=(n*1000).to_i
		n=n.to_f/1000      
	end #def round_3places
  
	def input_Offsets
		# open a file in read mode	       
		f = File.open(@file, "r")  
		ans = f.readlines
		# Close the file
		f.close
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			@cl_offset[i-1] = ans[7+i]
			prompts = ["CL Distance Domino " + i.to_s + " to Domino " + i2.to_s + " " ]
			defaults= [@cl_offset[i-1]]                     
			results = inputbox prompts, defaults, "Input CL Distance between Dominos"
			if !results then;Sketchup.send_action "selectSelectionTool:";return;end
			
			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(results[0])
			if !@cl_offset[i-1] then @cl_offset[i-1]=results[0] end; ans[7+i] = results[0]      
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0
		# Write an array of Input Data to the file
		# open a file in write mode	       
		f = File.open(@file, "w")
		f.puts ans
		# Close the file
		f.close       
	end #def input_Offsets 

	def cal_xCL_points(x1,e_len,dir_x)
		@xcl=Array.new @quanity.to_i
		case @spacing
			when "Symetrical"
				#Debug UI.messagebox("Spacing = " + @spacing.to_s)
				if @quanity == 1
					@xcl[0]=x1+(@end2CL*dir_x)
					#Debug UI.messagebox("@xcl[0] = " + @xcl[0].to_s)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity == 2
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)        
				elsif @quanity == 3
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len/2)*dir_x)
					@xcl[2]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity >= 4
					#Debug UI.messagebox("Started Quanity = " + @quanity.to_s)
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
					@xcl[@quanity/2]=x1+((e_len/2)*dir_x)
					n =(@quanity-2)/2  #loop number
					sn = 1 #Starting number
					i = 0
					while n > 0
						temp_offset=@cl_offset[i]
						#Debug  UI.messagebox("CL Offset = "+ temp_offset.to_s)
						@xcl[sn]=@xcl[sn-1]+((temp_offset)*dir_x)
						@xcl[@quanity-1-sn]=@xcl[@quanity-sn]-((temp_offset)*dir_x)
						sn = sn + 1
							n = n -1
						i = i + 1
					end # while n > 0                
				end #if @quanity == 
				#Debug  @xcl.each {|e| UI.messagebox("Symetrical CL point = "+ e.to_s)}           
			when "Equal"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
				@xcl[0]=x1+(@end2CL*dir_x)
				@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
				n =(@quanity-2)  #loop number
				sn = 1 #Starting number
				while n > 0
					@xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
					sn = sn + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
			when "User"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				@xcl[0]=x1+(@end2CL*dir_x)
				n =(@quanity-1)  #loop number
				sn = 1 #Starting number
				i = 0
				while n > 0
					user_offset=@cl_offset[i]
					#user_offset=user_offset.to_l
					#Debug UI.messagebox("user_offset = "+ user_offset.to_s)
					@xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
					sn = sn + 1
					i = i + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
		when "Fixed"
			#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
			@xcl[0]=x1+(@end2CL*dir_x)
			n =(@quanity-1)  #loop number
			sn = 1 #Starting number
			while n > 0
				fixed_offset=@cl_offset[0]
				@xcl[sn]=@xcl[sn-1]+((fixed_offset)*dir_x)
				sn = sn + 1
				n = n -1
			end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Fixed CL point = "+ e.to_s)}         
		end #case @spacing
		#Debug UI.messagebox("End Spacing = " + @spacing.to_s)
	end #cal_xCL_points

	def points_DominoMortise_Comp1(x,y,z,r)
		@pt=Array.new 14
		@pt[0]=Geom::Point3d.new((x+@xf)+r,y,z)
		@pt[1]=Geom::Point3d.new((x+@xf)+(0.866*r),y+(0.5*r),z)
		@pt[2]=Geom::Point3d.new((x+@xf)+(0.5*r),y+(0.866*r),z)
		@pt[3]=Geom::Point3d.new((x+@xf),y+r,z)
		@pt[4]=Geom::Point3d.new((x-@xf),y+r,z)
		@pt[5]=Geom::Point3d.new((x-@xf)-(0.5*r),y+(0.866*r),z)
		@pt[6]=Geom::Point3d.new((x-@xf)-(0.866*r),y+(0.5*r),z)
		@pt[7]=Geom::Point3d.new((x-@xf)-r,y,z)
		@pt[8]=Geom::Point3d.new((x-@xf)-(0.866*r),y-(0.5*r),z)
		@pt[9]=Geom::Point3d.new((x-@xf)-(0.5*r),y-(0.866*r),z)
		@pt[10]=Geom::Point3d.new((x-@xf),y-r,z)
		@pt[11]=Geom::Point3d.new((x+@xf),y-r,z)
		@pt[12]=Geom::Point3d.new((x+@xf)+(0.5*r),y-(0.866*r),z)
		@pt[13]=Geom::Point3d.new((x+@xf)+(0.866*r),y-(0.5*r),z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_DominoMortise_Comp1
  
	def points_DominoCL(x,y)
		@clpt=Array.new 4
		cline=0.03.inch
		@clpt[0]=Geom::Point3d.new(x+(cline),y,0)
		@clpt[1]=Geom::Point3d.new(x,y,0)
		@clpt[2]=Geom::Point3d.new(x,y+(cline),0)
		@clpt[3]=Geom::Point3d.new(x,y,0)
	end #def points_DominoCL	
    
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select 1st Component for Domino Joint:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select First Edge in component 1 Parallel to Domino Width:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities;
				@status=1; if @toolType == "SingleComp_Tool" then @status=2 end
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end # if@comp1			
		elsif @status==1	#select 2nd component
			ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
			if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp2); @ent2=@comp2.definition.entities;  @status=2
				@tform2 = @comp2.transformation.inverse
				onMouseMove(flags, x, y, view)
				@sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
			else
				UI.messagebox "Selection invalid, must be component"
			end #@entity			
		elsif @status==2 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@tform = @ip.transformation
					@edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position
					@status = 3
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid Use Right Click & Hide Interfering Comp"
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid "	
			end #if(@ip.valid?)
		elsif @status==3 #select short edge, thickness
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position 
					if @toolType != "SingleComp_Tool" then @comp2.hidden=false end
					if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
						orig=@e1sp
						p1=@e1sp
						p2=@e1ep
					end #if ((@e1sp==
					if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
						orig=@e1ep
						p2=@e1sp
						p1=@e1ep
					end  #if ((@e1ep==
					if orig==@e2sp
						p3=@e2ep
						else
						p3=@e2sp
					end #if orig              
					#find face that has edges 1 and 2
					face=@edge1.common_face @edge2
					if (face==nil)#14                    #get valid face
						@oops=4
					else 
						#Debug UI.messagebox"got valid face"
						#get points in local coord system for face
						#xaxis parallel to @edge1
						x,y,z=face.normal.axes
						#get vector parallel to @edge1              			         			          
						ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
						xx=ar.normalize
						yy=z.cross xx
						localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
						backt=Geom::Transformation.axes(orig,xx,yy,z)
						edges=face.outer_loop.edges
						eind=edges.index(@edge1)
						e_len=@edge1.length
						#Points on face in local coordinates
						p1=p1.transform(localt)
						p2=p2.transform(localt)
						p3=p3.transform(localt)           
						x1=p1.x
						x2=p2.x
						y1=p1.y
						y2=p3.y
						if x1>x2 then dir_x = -1 end
						if x1<x2 then dir_x = +1 end
						if y1<y2 then dir_y = +1 end
						if y1>y2 then dir_y = -1 end
						#Get domino data
						dominoDimensions
						r=@tt/2 #radius of Domino Slot
						self.cal_xCL_points(x1,e_len,dir_x)
						#Debug @xcl.each {|element| UI.messagebox("Cal @xcl Points = "+ element.to_s)}
                        ycl = y1+(@edge2CL*dir_y)
						#Debug UI.messagebox("yc1 = "+ ycl.to_s)    		        
						#START LOOP
						i=0
						n=@quanity
						Sketchup.active_model.start_operation "Domino Slot Face"
						while n > 0
							#Calculate points for Domino Slot Face							
							points_DominoMortise_Comp1(@xcl[i],ycl,0,r)
							#Calculate points for CLPoint of Domino Slot Face
							#Transform Domino Slot Face Points to Component 1 Coordinates
							@pt=@pt.collect {|point| point.transform(backt)}
							#Add Domino Slot Face
							if @toolType=="Joint_Tool" then @comp2.hidden=false  end
							@comp1.hidden=false
							@entity=@comp1
							codef=@entity.definition
							@sel.clear
							@sel.add(@comp1)
							cface=codef.entities.add_face(@pt) #Fixed array so it uses all elements!
							cface.pushpull(-1*@mdC1)
							#Feature added to First component at this point
							if @toolType=="Joint_Tool"
								@entity=@comp2
								codef=@entity.definition
								@sel.clear
								@sel.add(@comp2)
								@statusstring= "Adding Feature to Second Component"
								Sketchup::set_status_text @statusstring,SB_PROMPT                            	            
								@ptg=@pt.collect {|point| point.transform(@tform)}
								#Debug @ptg.each {|e| UI.messagebox("point c1g = "+ e.to_s)}
								@ptC2=@ptg.collect {|point| point.transform(@tform2)}
								#Debug @clptC2.each {|e| UI.messagebox("CL point c2 = "+ e.to_s)}	            
								cface=codef.entities.add_face(@ptC2)
								cface.pushpull(-1*@mdC2)
								#Feature added to second component at this point
							end #if @toolType=="Joint_Tool"
							i=i+1
							n=n-1
							#Debug  UI.messagebox("Number Holes Left = " + n.to_s )
						end #while n > 0
						Sketchup.active_model.commit_operation
						#END LOOP 
						result=UI.messagebox "Domino Slots Complete; Add Dominos to Slots? ", MB_YESNO              
						if result==6  #Add Dominos to  slots
							Sketchup.active_model.start_operation "Dominos"
							@widthSetting = "Standard" #Use for Dominos
							dominoDimensions
							i=0
							n=@quanity
							while n > 0
								@sel.clear
								model = Sketchup.active_model
								entities = model.active_entities
								group = entities.add_group
								entities = group.entities
								# Calculate New Points for Creating Dominos
								points_DominoMortise_Comp1(@xcl[i],ycl,0,r) #@pt
								@pt=@pt.collect {|point| point.transform(backt)} #Transform to Comp 1
								@pt=@pt.collect {|point| point.transform(@tform)} #Transform to Global
								#Add Dominos
								dface=entities.add_face(@pt) 
								dface.pushpull(-1*@mdC1)
								dface.pushpull(@mdC2)
								# Convert the Group to a Component 
								component_instance=group.to_component
								component_instance.definition.name="Part Domino_" + @dominoSize
								i=i+1
								n=n-1	
							end #while n loop
							Sketchup.active_model.commit_operation
						end #result==6
						#Program is Complete and ends
						Sketchup.send_action "selectSelectionTool:"
					end #face==nil                        
				end	#if(@ip.edge &&
			else
				UI.messagebox "Selection invalid"	
			end #if(@ip.valid?)	
		end #elsif @status==
	end # def onLButtonDown
	
	def onRButtonDown(flags,x,y,view)
		reset
		self.dialog
		self.no_of_Offset_Input
		if @noOffsets >= 1
			input_Offsets
		end
	end

	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
end #end class K2_Tools_DominoJoint

class K2_Tools_Domino  < K2_Tools_DominoJoint
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @toolType = "SingleComp_Tool"
	end
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select Component for Domino Mortise:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select First Edge in component Parallel to Domino Width:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 90 degrees to First Edge (defines End to CL):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end

end # class K2_Tools_Domino
 


 file_loaded("K2_Tools_DominoJoint.rb")
 
