#
#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # V2.0
  # Name :        K2_TenonTool.rb  ###
  # Date :	      Aug 2011
  # Description : Removed CL Marks
  #
  # Author :       Keith Krueger & Sam Mitchel (with a big help from plugin forum:)
  #
 
require 'sketchup.rb'
require 'K2WS_Tools/KK_Input_parsing.rb'
 
class K2_M_T_Joint  
   include KK_InputParsing
    
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@inpt = Sketchup::InputPoint.new		
		@tinpt= Sketchup::InputPoint.new
		@tooltype == "Joint_tool"
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsM_TJointHelp.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			reset
			self.dialog
			if @edge == "Haunched" then self.haunchedData end
			if @toolType != "SingleComp_Tool" then self.mortiseDepth end
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				reset
				self.dialog
				if @edge == "Haunched" then self.haunchedData end
				if @toolType != "SingleComp_Tool" then self.mortiseDepth end
			else
				reset
				self.readExistingData
			end
		end
		
	end
	
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false
	end
	
	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close		
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end
	
	def readExistingData
		self.read_data
		
		@edge=@ans[0].to_s.strip; 
		
		@todist=KK_InputParsing.string_to_length_formula(@ans[1].to_s.strip)
		if !@todist then @todist=@ans[1].to_l end;
                                
        @thickness=KK_InputParsing.string_to_length_formula(@ans[2].to_s.strip)
		if !@thickness then @thickness=@ans[2].to_l end; 
        
        @length=KK_InputParsing.string_to_length_formula(@ans[3].to_s.strip)
		if !@length then @length=@ans[3].to_l end; 
        
		@ed1toend=KK_InputParsing.string_to_length_formula(@ans[4].to_s.strip)
		if !@ed1toend then @ed1toend=@ans[4].to_l end; 
        
		@ed2toend=KK_InputParsing.string_to_length_formula(@ans[5].to_s.strip)
		if !@ed2toend then @ed2toend=@ans[5].to_l end;

		if @edge == "Haunched"
			@haunch=@ans[6].to_s.strip;
		
			@hlength=KK_InputParsing.string_to_length_formula(@ans[7].to_s.strip)
			if !@hlength then @hlength=@ans[7].to_l end;
		end
        if @toolType != "SingleComp_Tool" 
			@mdepth=KK_InputParsing.string_to_length_formula(@ans[8].to_s.strip)
			if !@mdepth then @mdepth=@ans[8].to_l end;
		end
	end #readExistingData
	
	def dialog
		# open input data file in read mode	       
		self.read_data
		prompts = ["End Shape","Edge1 to Tenon", "Thickness","Length","End1 Shoulder","End2 Shoulder"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip,@ans[4].to_s.strip,@ans[5].to_s.strip]
		choices = ["Square|Round|Haunched","","","","",""]  
		results = inputbox prompts, defaults, choices, "Input Data for Tenon "

		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@edge=results[0].to_s.strip; @ans[0] = results[0]
		
		@todist=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
		if !@todist then @todist=results[1].to_l end; @ans[1] = results[1]
                                
        @thickness=KK_InputParsing.string_to_length_formula(results[2].to_s.strip)
		if !@thickness then @thickness=results[2].to_l end; @ans[2] = results[2]
        
        @length=KK_InputParsing.string_to_length_formula(results[3].to_s.strip)
		if !@length then @length=results[3].to_l end; @ans[3] = results[3]
        
		@ed1toend=KK_InputParsing.string_to_length_formula(results[4].to_s.strip)
		if !@ed1toend then @ed1toend=results[4].to_l end; @ans[4] = results[4]
        
		@ed2toend=KK_InputParsing.string_to_length_formula(results[5].to_s.strip)
		
		if !@ed2toend then @ed2toend=results[5].to_l end; @ans[5] = results[5]
		
		# Write an array of Input Data to the file
		self.write_data
	end #dialog
	
	
	def haunchedData
		# open input data file in read mode	       
		self.read_data
		prompts = ["End Shape","Haunch Length"]
        defaults= [@ans[6].to_s.strip,@ans[7].to_s.strip]              
		choices = ["Shoulder 1|Shoulder 2|Both",""]   
		results = inputbox prompts, defaults, choices, "Haunched Tenon Dim"
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@haunch=results[0].to_s.strip; @ans[6] = results[0]
		
		@hlength=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
		if !@hlength then @hlength=results[1].to_l end; @ans[7] = results[1]
		
		# Write an array of Input Data to the file
		self.write_data
	end #haunchedData
	
	def mortiseDepth
		# open input data file in read mode	       
		self.read_data
		prompts = ["Mortise Depth COMPONENT 2 "]
        defaults= [@ans[8].to_s.strip]                
		results = inputbox prompts, defaults, "Mortise Depth COMPONENT 2"
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		
		@mdepth=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
		if !@mdepth then @mdepth=results[0].to_l end; @ans[8] = results[0]
		# open input data file in write mode	       
		self.write_data
	end #mortiseDepth	
	
	def points_TenonSq(x1,x2,y1,z,dir_x,dir_y)
		@pt=Array.new 4
		@pt[0]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist*dir_y),z)
		@pt[1]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
		@pt[2]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
		@pt[3]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist*dir_y),z)       
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #def points_TenonSq  
    
	def points_TenonRd(x1,x2,y1,z,dir_x,dir_y)
		@pt=Array.new 14
		r=(@thickness/2).to_l
		@pt[0]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist*dir_y),z)
		@pt[1]=Geom::Point3d.new(@pt[0].x-(0.5*r*dir_x),@pt[0].y+(r-0.866*r)*dir_y,z)
		@pt[2]=Geom::Point3d.new(@pt[0].x-(0.866*r*dir_x),@pt[0].y+(r-0.5*r)*dir_y,z)
		@pt[3]=Geom::Point3d.new(@pt[0].x-(r*dir_x),@pt[0].y+(r*dir_y),z)
		@pt[4]=Geom::Point3d.new(@pt[2].x,@pt[3].y+(0.5*r*dir_y),z)
		@pt[5]=Geom::Point3d.new(@pt[1].x,@pt[3].y+(0.866*r*dir_y),z)
		@pt[6]=Geom::Point3d.new(@pt[0].x,@pt[3].y+(r*dir_y),z)
		@pt[7]=Geom::Point3d.new(x2-(@ed2toend*dir_x),@pt[6].y,z)
		@pt[8]=Geom::Point3d.new(@pt[7].x+(0.5*r*dir_x),@pt[5].y,z)
		@pt[9]=Geom::Point3d.new(@pt[7].x+(0.866*r*dir_x),@pt[4].y,z)
		@pt[10]=Geom::Point3d.new(@pt[7].x+(r*dir_x),@pt[3].y,z)
		@pt[11]=Geom::Point3d.new(@pt[9].x,@pt[2].y,z)
		@pt[12]=Geom::Point3d.new(@pt[8].x,@pt[1].y,z)
		@pt[13]=Geom::Point3d.new(@pt[7].x,@pt[0].y,z)                      
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #def points_TenonRd
	
	def points_MortiseCL(x1,x2,y1,dir_y)
		@clpt=Array.new 4
		cline=0.03.inch
		x=@ed1toend+((x2-x1-@ed1toend-@ed2toend)/2)
		y=(y1+@todist+(@thickness/2))*dir_y
		@clpt[0]=Geom::Point3d.new(x+(cline),y,0)
		@clpt[1]=Geom::Point3d.new(x,y,0)
		@clpt[2]=Geom::Point3d.new(x,y+(cline),0)
		@clpt[3]=Geom::Point3d.new(x,y,0)
	end #def points_DominoCL
		
	def points_HaunchedTenon1(x1,x2,y1,z,dir_x,dir_y)
		@pth=Array.new 4
		case @haunch
			when "Shoulder 1"
				@pth[0]=Geom::Point3d.new(x1,y1+(@todist*dir_y),z)
				@pth[1]=Geom::Point3d.new(x1,y1+(@todist+@thickness)*dir_y,z)
				@pth[2]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
				@pth[3]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist*dir_y),z)       
				#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}
			when "Shoulder 2"
				@pth[0]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist*dir_y),z)
				@pth[1]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
				@pth[2]=Geom::Point3d.new(x2,y1+(@todist+@thickness)*dir_y,z)
				@pth[3]=Geom::Point3d.new(x2,y1+(@todist*dir_y),z) 
			when "Both"
				@pth[0]=Geom::Point3d.new(x1,y1+(@todist*dir_y),z)
				@pth[1]=Geom::Point3d.new(x1,y1+(@todist+@thickness)*dir_y,z)
				@pth[2]=Geom::Point3d.new(x2,y1+(@todist+@thickness)*dir_y,z)
				@pth[3]=Geom::Point3d.new(x2,y1+(@todist*dir_y),z)
		end
		#Debug @pth.each {|e| UI.messagebox("point = "+ e.to_s)}
	end #def points_HaunchedTenon1 
	
	def points_HaunchedTenon2(x1,x2,y1,z,dir_x,dir_y)
		@pt=Array.new 4
		@pt[0]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist*dir_y),z)
		@pt[1]=Geom::Point3d.new(x1+(@ed1toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
		@pt[2]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist+@thickness)*dir_y,z)
		@pt[3]=Geom::Point3d.new(x2-(@ed2toend*dir_x),y1+(@todist*dir_y),z)       
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #def points_HaunchedTenon2 
 
    
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select 1st Component for Tenon Joint:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select 2nd Component for joint:   [SPACE BAR] to Exit", SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select First Edge in component 1 Parallel to Tenon Width:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 1 90 degrees to First Edge (defines shoulder 1):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities; @status=1
				if @toolType == "SingleComp_Tool" then @status=2 end
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end # if@comp1			
		elsif @status==1	#select 2nd component
			ph=view.pick_helper; ph.do_pick x,y; @comp2=ph.best_picked
			if @comp2 && (@comp2.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp2); @ent2=@comp2.definition.entities;  @status=2
				@tform2 = @comp2.transformation.inverse
				onMouseMove(flags, x, y, view)
				@sel.clear; @entity=@comp1; @sel.add(@comp1); @comp2.hidden=true
			else
				UI.messagebox "Selection invalid, must be component"
			end #@entity			
		elsif @status==2 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@tform = @ip.transformation
					@edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position
					@status = 3
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid "
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid Use Right Click & Hide Interfering Comp"	
			end #if(@ip.valid?)
		elsif @status==3 #select short edge, thickness
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position 
					if @toolType != "SingleComp_Tool" then @comp2.hidden=false end
					if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
						orig=@e1sp
						p1=@e1sp
						p2=@e1ep
					end #if ((@e1sp==
					if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
						orig=@e1ep
						p2=@e1sp
						p1=@e1ep
					end  #if ((@e1ep==
					if orig==@e2sp
						p3=@e2ep
					else
						p3=@e2sp
					end #if orig              
					#find face that has edges 1 and 2
					face=@edge1.common_face @edge2
					if (face==nil)#14                    #get valid face
						UI.messagebox"face not valid"
					else #14
						#Debug UI.messagebox"got valid face######################"
						#get points in local coord system for face
						#xaxis parallel to @edge1
						x,y,z=face.normal.axes
						#get vector parallel to @edge1               			          
						ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
						xx=ar.normalize
						yy=z.cross xx
						localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
						backt=Geom::Transformation.axes(orig,xx,yy,z)
						edges=face.outer_loop.edges
						eind=edges.index(@edge1)
						e_len=@edge1.length
						#Points on face in local coordinates
						p1=p1.transform(localt); p2=p2.transform(localt); p3=p3.transform(localt)			      			                 
						x1=p1.x; x2=p2.x; y1=p1.y; y2=p3.y			      			      			      
						dir_x = +1; if x1>x2 then dir_x = -1 end
						dir_y = +1; if y1>y2 then dir_y = -1 end
					  
						#Get points for Tenon in Local Coordinate System  points_HaunchedTenon2
						if @edge == "Square" then points_TenonSq(x1,x2,y1,0,dir_x,dir_y) end #Returns @pt[] 4 points
						if @edge == "Round" then points_TenonRd(x1,x2,y1,0,dir_x,dir_y) end #Returns @pt[] 12 points
						if @edge == "Haunched" then points_HaunchedTenon1(x1,x2,y1,0,dir_x,dir_y) end #Returns @pth[] 4 points
						if @edge == "Haunched" then points_HaunchedTenon2(x1,x2,y1,@hlength,dir_x,dir_y) end #Returns @pt[] 4 points
						#Transform Points to Component 1 Coordinates and add Mortise & CL
						@pt=@pt.collect {|point| point.transform(backt)}
						codef=@comp1.definition
						@sel.clear
						@sel.add(@comp1)
						Sketchup.active_model.start_operation "Add Mortise"
						if @edge == "Haunched"
							@pth=@pth.collect {|point| point.transform(backt)}
							cface=codef.entities.add_face(@pth)
							cface.pushpull(@hlength)
							cface=codef.entities.add_face(@pt)
							cface.pushpull((@length-@hlength))
						else
							cface=codef.entities.add_face(@pt) #Fixed array so it uses all elements!
							cface.pushpull(@length)																
						end	  						
						if @toolType != "SingleComp_Tool"
							#if @edge == "Haunched" then @pt.each{|pt|pt.z = -1 * pt.z} end # Reverse direction of points for Comp2	
							#Transform Mortise Points to Global Coordinates 
							@pt=@pt.collect {|point| point.transform(@tform)}
							#Transform to Component 2
							@pt=@pt.collect {|point| point.transform(@tform2)}
							codef=@comp2.definition
							@sel.clear
							@sel.add(@comp2)
							#Add Mortise
							if @edge == "Haunched"
								@pth=@pth.collect {|point| point.transform(@tform)}
								@pth=@pth.collect {|point| point.transform(@tform2)}
								cface=codef.entities.add_face(@pth)
								cface.pushpull(-1*@hlength)
								cface=codef.entities.add_face(@pt)
								cface.pushpull(-1*(@mdepth-@hlength))
							else	
								cface=codef.entities.add_face(@pt)
								cface.pushpull(-1*@mdepth)								
							end
							end
						Sketchup.active_model.commit_operation
						#Program is Complete and ends
						Sketchup.send_action "selectSelectionTool:" 
					end #14
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid"	
			end #if(@ip.valid?)	
		end #elsif @status==
	end # def onLButtonDown
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
	def onRButtonDown(flags,x,y,view)
		reset
		self.dialog
		if @edge == "Haunched" then self.haunchedData end
		if @toolType != "SingleComp_Tool" then self.mortiseDepth end
	end

end #end class K2_M_T_Joint

class K2_TenonTool  < K2_M_T_Joint

	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false; @toolType = "SingleComp_Tool"
	end
	
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select Component for Tenon:   RIGHT CLICK to Change Data:   [SPACE BAR] to Exit", SB_PROMPT) if @status==0 
		Sketchup::set_status_text("Select First Edge in component Parallel to Tenon Width:   [SPACE BAR] to Exit",SB_PROMPT) if @status==2
		Sketchup::set_status_text("Select Second Edge in component 90 degrees to First Edge (defines shoulder 1):   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end

end # class K2_TenonTool
   


 file_loaded("K2_ToolsM_TJoint.rb")
 
