


#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
  #portions of this code are modifications of examples from Google
  # Copyright 2005-2007, Google, Inc.
  # V2.0
  # Name :        K2_Tools_ScrewHoles.rb  ###
  # Date :	      Aug 2011
  # Description : Places multiple screw holes in face of component with cross lines @ center
  #               C'Sunk, C'Bore, Straight, CLs Only
  #				  Removed CL Points; Fixed Spacing < 2
  # Author :       Keith Krueger (with a big help from plugin forum:)
  #
 
require 'sketchup.rb'
require  'K2WS_Tools/KK_Input_parsing.rb'
 
class K2_Tools_ScrewHoles  
   include KK_InputParsing
    
	def initialize
		@mod = Sketchup.active_model
		@sel = @mod.selection
		@ip = Sketchup::InputPoint.new
		@base_path = Sketchup.find_support_file("Plugins")
		@rel_path = "K2WS_Tools"
		help_path = File.join(@base_path,@rel_path)
		@helpfile = "K2_ToolsScrewHoles.jpg"
		fullpath = File.join(@base_path,@rel_path,@helpfile)
		@file =  __FILE__.gsub(/.rb$/,'.txt')
		if !(FileTest.exist?(@file)) 
			f = File.new(@file, "w+") 
			f.close
			UI.openURL( fullpath )
			UI.messagebox("If this Came up without you seeing the HELP PICTURE: Check Out the Help File Picture in your Browser or look in:  " + help_path + "  You can cycle thru all the help pictures to get a better idea of how the program works.")
			self.reset
			self.data_Input
			self.no_of_Offset_Input
			if @noOffsets >= 1
				self.input_Offsets
			end
		else
			result=UI.messagebox "Review or Change Data? ", MB_YESNO
			if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  
				self.reset
				self.data_Input
				self.no_of_Offset_Input
				if @noOffsets >= 1
					input_Offsets
				end
			else
				self.reset
				self.readExistingData
			end
		end
		
	end
	
	def reset
		@best = nil; @comp1 = nil; @comp2 = nil; @sel.clear; @status = 0; @drawn = false
	end

	def read_data
		# Read an array of Input Data from the file 
		# open input data file in read mode	       
		f = File.open(@file, "r")    
		@ans = f.readlines
		# Close the file
		f.close		
	end
	
	def write_data
		# Write an array of Input Data to the file
		# open input data file in write mode	       
		f = File.open(@file, "w")
		f.puts @ans
		# Close the file
		f.close
	end

	def readExistingData
        self.read_data
		
		@type=@ans[0].to_s.strip
		
		@edge2CL=KK_InputParsing.string_to_float_formula(@ans[1].to_s.strip)
		if !@edge2CL then @edge2CL=@ans[1] end; 
		
		@end2CL=KK_InputParsing.string_to_length_formula(@ans[2].to_s.strip)
		if !@end2CL then @end2CL=@ans[2] end; 
		
		@quanity=KK_InputParsing.string_to_integer_formula(@ans[3].to_s.strip)		
		if !@quanity then @quanity=@ans[3].to_i end;

		@feature=@ans[4].to_s.strip;
		
		@h_dia=KK_InputParsing.string_to_length_formula(@ans[5].to_s.strip)
		if !@h_dia then @h_dia=@ans[5] end;
		
		@s_dia=KK_InputParsing.string_to_length_formula(@ans[6].to_s.strip)
		if !@s_dia then @s_dia=@ans[6] end;
		
		@ang_deg=KK_InputParsing.string_to_integer(@ans[7].to_s.strip)
		if !@ang_deg then @ang_deg=@ans[7] end;
		
		@cb_dep=KK_InputParsing.string_to_length_formula(@ans[8].to_s.strip)
		if !@cb_dep then @cb_dep=@ans[8] end; 
		
		@spacing=@ans[9].to_s.strip
		
		if @type == "C_Sunk" then	@z1 = (((@h_dia-@s_dia)/2)/ Math.tan((@ang_deg/2).degrees)).to_l end
		if @type == "C_Bore"  then	@z1 = @cb_dep.to_l  end
		
		self.no_of_Offset_Input
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(@ans[9+i].to_s)
			if !@cl_offset[i-1] then @cl_offset[i-1]=@ans[9+i] end;           
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0	
	end #readExistingData
	
	def data_Input
		self.read_data
		#Type and Center Line Data
		prompts = ["Type of Hole","Edge to CL of Hole ","End to CL","Quanity","Spacing"]
		defaults= [@ans[0].to_s.strip,@ans[1].to_s.strip,@ans[2].to_s.strip,@ans[3].to_s.strip,@ans[9].to_s.strip]
		choices = ["C_Sunk|C_Bore|Straight","","","","Symetrical|Equal|User|Fixed"] 
		results = inputbox prompts, defaults, choices, "Input Hole Type & CL Data"
		if !results then;Sketchup.send_action "selectSelectionTool:";return;end
		@type=results[0].to_s.strip;  @ans[0] = results[0]
		
		@edge2CL=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
		if !@edge2CL then @edge2CL=results[1].to_l end; @ans[1] = results[1]
                
		@end2CL=KK_InputParsing.string_to_length_formula(results[2].to_s.strip)
		if !@end2CL then @end2CL=results[2].to_l end;  @ans[2] = results[2]
		
		@quanity=KK_InputParsing.string_to_integer_formula(results[3].to_s.strip)
		if !@quanity then @quanity=results[3].to_i end; @ans[3] = results[3]
		
		@spacing=results[4].to_s.strip; @ans[9] = results[4]		
		self.write_data
		
		self.read_data
		if @type == "C_Sunk" #Counter Sink Data
			prompts = ["Screw Hd Dia ","Screw Dia","CS Angle (Deg)"]
			defaults= [@ans[5].to_s.strip,@ans[6].to_s.strip,@ans[7].to_s.strip]
			choices = ["","",""]
			results = inputbox prompts, defaults, choices, "Counter Sink Data"
			if !results then;Sketchup.send_action "selectSelectionTool:";return;end  
			@feature="All Features"; @ans[4] = "All Features"
					
			@h_dia=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
			if !@h_dia then @h_dia=results[0].to_l end; @ans[5] = results[0]
					
			@s_dia=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
			if !@s_dia then @s_dia=results[1].to_l end; @ans[6] = results[1]
			
			@ang_deg=KK_InputParsing.string_to_integer(results[2].to_s.strip)
			if !@ang_deg then @ang_deg=results[2].to_i end; @ans[7] = results[2]
			
		end # @type == "C_Sunk"
		if @type == "C_Bore"  #Counter Bore Data
			prompts = ["Screw Hd Dia ","Screw Dia","CS Depth"]
			defaults= [@ans[5].to_s.strip,@ans[6].to_s.strip,@ans[8].to_s.strip]
			choices = ["","",""]
			results = inputbox prompts, defaults, choices, "Counter Bore Data"
			if !results then;Sketchup.send_action "selectSelectionTool:";return;end  
			@feature="All Features"; @ans[4] = "All Features"
					
			@h_dia=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
			if !@h_dia then @h_dia=results[0].to_l end; @ans[5] = results[0]
					
			@s_dia=KK_InputParsing.string_to_length_formula(results[1].to_s.strip)
			if !@s_dia then @s_dia=results[1].to_l end;  @ans[6] = results[1]
			
			@cb_dep=KK_InputParsing.string_to_length_formula(results[2].to_s.strip)
			if !@cb_dep then @cb_dep=results[2].to_l end; @ans[8] = results[2]
		end # @type == "C_Bore"	
		if @type == "Straight"  #Straight Hole Data
			prompts = ["Screw Dia"]
			defaults= [@ans[6].to_s.strip]
			choices = [""]
			results = inputbox prompts, defaults, choices, "Straight Hole Data"
			if !results then;Sketchup.send_action "selectSelectionTool:";return;end  
			@feature="All Features"; @ans[4] = "All Features"
					
			@s_dia=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
			if !@s_dia then @s_dia=results[0].to_l end; @ans[6] = results[0]
			
		end #@type == "Straight
		self.write_data
		if @type == "C_Sunk" then	@z1 = (((@h_dia-@s_dia)/2)/ Math.tan((@ang_deg/2).degrees)).to_l end
		if @type == "C_Bore"  then	@z1 = @cb_dep.to_l  end
	end #data_Input
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end

	def no_of_Offset_Input
		case @spacing
			when "Symetrical"
			if @quanity <= 3
				@noOffsets=0
			else   
				@noOffsets=(@quanity - 2)/2
			end #if @quanity <= 3      
			when "Equal"
				@noOffsets=0
			when "User"
				@noOffsets=@quanity - 1
			when "Fixed"
				if @quanity <= 1
					@noOffsets=0
				else   
					@noOffsets=1
				end
		end #case @spacing
	end #no_of_offsets
      
	def round_3places(n)  
		n=(n*1000).to_i
		n=n.to_f/1000      
	end #def round_3places
  
	def input_Offsets
		# open a file in read mode	       
		self.read_data
		n = @noOffsets
		@cl_offset=Array.new n
		i = 1
		i2 = 2
		while n > 0
			#@cl_offset[i-1] = @ans[9+i]
			prompts = ["CL Distance Screw " + i.to_s + " to Screw " + i2.to_s + " " ]
     
			defaults= [@ans[9+i].to_s.strip]
                       
			results = inputbox prompts, defaults, "Input CL Distance between Screws"

			@cl_offset[i-1]=KK_InputParsing.string_to_length_formula(results[0].to_s.strip)
			if !@cl_offset[i-1] then @cl_offset[i-1]=results[0] end; @ans[9+i] = results[0]       
			n = n-1
			i = i+1
			i2 = i2+1		
		end #while n > 0
		# Write an array of Input Data to the file
		self.write_data     
	end #def input_Offsets 

	def cal_xCL_points(x1,e_len,dir_x)
		@xcl=Array.new @quanity.to_i
		@end2CL=@end2CL.to_l
		case @spacing
			when "Symetrical"
				#Debug UI.messagebox("Spacing = " + @spacing.to_s)
				if @quanity == 1
					@xcl[0]=x1+(@end2CL*dir_x)
					#Debug UI.messagebox("@xcl[0] = " + @xcl[0].to_s)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity == 2
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)        
				elsif @quanity == 3
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[1]=x1+((e_len/2)*dir_x)
					@xcl[2]=x1+((e_len-@end2CL)*dir_x)
					#Debug UI.messagebox("Completed Quanity = " + @quanity.to_s)
				elsif @quanity >= 4
					#Debug UI.messagebox("Started Quanity = " + @quanity.to_s)
					@xcl[0]=x1+(@end2CL*dir_x)
					@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
					@xcl[@quanity/2]=x1+((e_len/2)*dir_x)
					n =(@quanity-2)/2  #loop number
					sn = 1 #Starting number
					i = 0
					while n > 0
						temp_offset=@cl_offset[i]
						#Debug  UI.messagebox("CL Offset = "+ temp_offset.to_s)
						@xcl[sn]=@xcl[sn-1]+((temp_offset)*dir_x)
						@xcl[@quanity-1-sn]=@xcl[@quanity-sn]-((temp_offset)*dir_x)
						sn = sn + 1
							n = n -1
						i = i + 1
					end # while n > 0                
				end #if @quanity == 
				#Debug  @xcl.each {|e| UI.messagebox("Symetrical CL point = "+ e.to_s)}           
			when "Equal"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				equal_offset=(e_len-(@end2CL*2))/(@quanity-1)
				@xcl[0]=x1+(@end2CL*dir_x)
				@xcl[@quanity-1]=x1+((e_len-@end2CL)*dir_x)
				n =(@quanity-2)  #loop number
				sn = 1 #Starting number
				while n > 0
					@xcl[sn]=@xcl[sn-1]+((equal_offset)*dir_x)
					sn = sn + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Equal CL point = "+ e.to_s)}        
			when "User"
				#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
				@xcl[0]=x1+(@end2CL*dir_x)
				n =(@quanity-1)  #loop number
				sn = 1 #Starting number
				i = 0
				while n > 0
					user_offset=@cl_offset[i]
					#user_offset=user_offset.to_l
					#Debug UI.messagebox("user_offset = "+ user_offset.to_s)
					@xcl[sn]=@xcl[sn-1]+((user_offset)*dir_x)
					sn = sn + 1
					i = i + 1
					n = n -1
				end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("User CL point = "+ e.to_s)} 
		when "Fixed"
			#Debug UI.messagebox("Case Spacing = " + @spacing.to_s)
			@xcl[0]=x1+(@end2CL*dir_x)
			n =(@quanity-1)  #loop number
			sn = 1 #Starting number
			while n > 0
				fixed_offset=@cl_offset[0]
				@xcl[sn]=@xcl[sn-1]+((fixed_offset)*dir_x)
				sn = sn + 1
				n = n -1
			end # while n > 0
			#Debug @xcl.each {|e| UI.messagebox("Fixed CL point = "+ e.to_s)}         
		end #case @spacing
		#Debug UI.messagebox("End Spacing = " + @spacing.to_s)
	end #cal_xCL_points
	
	def points_ScrewHead(x,y,z,r,dir_y)
		@pthd=Array.new 12
		@pthd[0]=Geom::Point3d.new(x+r,y,z)
		@pthd[1]=Geom::Point3d.new(x+(0.866*r),y+(0.5*r)*dir_y,z)
		@pthd[2]=Geom::Point3d.new(x+(0.5*r),y+(0.866*r)*dir_y,z)
		@pthd[3]=Geom::Point3d.new(x,y+r*dir_y,z)
		@pthd[4]=Geom::Point3d.new(x-(0.5*r),y+(0.866*r)*dir_y,z)
		@pthd[5]=Geom::Point3d.new(x-(0.866*r),y+(0.5*r)*dir_y,z)
		@pthd[6]=Geom::Point3d.new(x-r,y,z)
		@pthd[7]=Geom::Point3d.new(x-(0.866*r),y-(0.5*r)*dir_y,z)
		@pthd[8]=Geom::Point3d.new(x-(0.5*r),y-(0.866*r)*dir_y,z)
		@pthd[9]=Geom::Point3d.new(x,y-r*dir_y,z)
		@pthd[10]=Geom::Point3d.new(x+(0.5*r),y-(0.866*r)*dir_y,z)
		@pthd[11]=Geom::Point3d.new(x+(0.866*r),y-(0.5*r)*dir_y,z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_ScrewHead
	
	def points_Screw(x,y,z,r,dir_y)
		@ptsc=Array.new 12
		@ptsc[0]=Geom::Point3d.new(x+r,y,z)
		@ptsc[1]=Geom::Point3d.new(x+(0.866*r),y+(0.5*r)*dir_y,z)
		@ptsc[2]=Geom::Point3d.new(x+(0.5*r),y+(0.866*r)*dir_y,z)
		@ptsc[3]=Geom::Point3d.new(x,y+r*dir_y,z)
		@ptsc[4]=Geom::Point3d.new(x-(0.5*r),y+(0.866*r)*dir_y,z)
		@ptsc[5]=Geom::Point3d.new(x-(0.866*r),y+(0.5*r)*dir_y,z)
		@ptsc[6]=Geom::Point3d.new(x-r,y,z)
		@ptsc[7]=Geom::Point3d.new(x-(0.866*r),y-(0.5*r)*dir_y,z)
		@ptsc[8]=Geom::Point3d.new(x-(0.5*r),y-(0.866*r)*dir_y,z)
		@ptsc[9]=Geom::Point3d.new(x,y-r*dir_y,z)
		@ptsc[10]=Geom::Point3d.new(x+(0.5*r),y-(0.866*r)*dir_y,z)
		@ptsc[11]=Geom::Point3d.new(x+(0.866*r),y-(0.5*r)*dir_y,z)   
		#Debug @pt.each {|e| UI.messagebox("point = "+ e.to_s)}    
	end #points_Screw
	
	def points_HolelCL(x,y)
		@clpt=Array.new 4
		@clpt[0]=Geom::Point3d.new(x+0.05.to_l,y,0)
		@clpt[1]=Geom::Point3d.new(x,y,0)
		@clpt[2]=Geom::Point3d.new(x,y+0.05.to_l,0)
		@clpt[3]=Geom::Point3d.new(x,y,0)
	end #def points_HolelCL
    
	def onMouseMove(flags, x, y, view)
		Sketchup::set_status_text("Select Component for Hole Detail:   RIGHT CLICK to Change Data:   SPACE BAR to Exit", SB_PROMPT) if @status==0
		Sketchup::set_status_text("Select THICKNESS EDGE:   [SPACE BAR] to Exit",SB_PROMPT) if @status==1
		Sketchup::set_status_text("Select EDGE in component to Define EDGE to CL Parallel to Pattern:   [SPACE BAR] to Exit",SB_PROMPT)  if @status==2
		Sketchup::set_status_text("Select EDGE in component 90 degrees to Previous Edge to Define end to CL:   [SPACE BAR] to Exit",SB_PROMPT) if @status==3
		@ip.pick view,x,y
		view.tooltip = @ip.tooltip     
		view.refresh
	end
	
	def onLButtonDown(flags, x, y, view)
		if @status==0 #select 1st component
			ph=view.pick_helper; ph.do_pick x,y; @comp1=ph.best_picked
			if @comp1 && (@comp1.is_a?(Sketchup::ComponentInstance))
				@sel.add(@comp1); @ent1=@comp1.definition.entities; @status=1
				onMouseMove(flags, x, y, view)
			else
				UI.messagebox "Selection invalid, must be component"
			end # if@comp1			
		elsif @status==1 #select thickness edge  @edge3
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge3 = @ip.edge; @edge3L = @edge3.length; @status = 2
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid "
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid "	
			end #if(@ip.valid?)			
		elsif @status==2 #select long edge, width
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@tform = @ip.transformation
					@edge1 = @ip.edge; @e1sp=@edge1.start.position; @e1ep=@edge1.end.position
					@status = 3
					onMouseMove(flags, x, y, view)
				else
					UI.messagebox "Selection invalid "
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid "	
			end #if(@ip.valid?)
		elsif @status==3 #select edge, 90 degrees to First Edge
			@ip.pick view,x,y
			if(@ip.valid?)
				if(@ip.edge && @ent1.include?(@ip.edge))
					@edge2 = @ip.edge; @e2sp=@edge2.start.position; @e2ep=@edge2.end.position 
					if ((@e1sp==@e2sp) || (@e1sp==@e2ep))
						orig=@e1sp
						p1=@e1sp
						p2=@e1ep
					end #if ((@e1sp==
					if ((@e1ep==@e2sp) || (@e1ep==@e2ep))
						orig=@e1ep
						p2=@e1sp
						p1=@e1ep
					end  #if ((@e1ep==
					if orig==@e2sp
						p3=@e2ep
					else
						p3=@e2sp
					end #if orig              
					#find face that has edges 1 and 2
					face=@edge1.common_face @edge2
					if (face==nil)#14                    #get valid face
						UI.messagebox "Selection invalid "
					else 
						#Debug UI.messagebox"got valid face"
						#get points in local coord system for face
						#xaxis parallel to @edge1
						x,y,z=face.normal.axes
						#get vector parallel to @edge1              			         			          
						ar=[p2.x-p1.x,p2.y-p1.y,p2.z-p1.z]
						xx=ar.normalize
						yy=z.cross xx
						localt=Geom::Transformation.axes(orig,xx,yy,z).invert!
						backt=Geom::Transformation.axes(orig,xx,yy,z)
						edges=face.outer_loop.edges
						eind=edges.index(@edge1)
						e_len=@edge1.length
						#Points on face in local coordinates
						p1=p1.transform(localt); p2=p2.transform(localt); p3=p3.transform(localt) 												           
						x1=p1.x; x2=p2.x; y1=p1.y; y2=p3.y 						
						if x1>x2 then dir_x = -1 end; if x1<x2 then dir_x = +1 end						
						if y1<y2 then dir_y = +1 end; if y1>y2 then dir_y = -1 end 
						#UI.messagebox("dir y = " + dir_y.to_s)						#Calculate x and y Center line Points 
						ycl = y1+(@edge2CL.to_l*dir_y)
						self.cal_xCL_points(x1,e_len,dir_x)						
						Sketchup.active_model.start_operation "Dowel Holes"                
						@entity=@comp1
						codef=@entity.definition
						@sel.clear
						@sel.add(@comp1)
						#START LOOP
						i=0
						n=@quanity
						while n > 0
							if @feature == "1 Feature Only" && n == @quanity then n=0 end #One feature only!
							case @type
								when "C_Sunk"
									#Calculate points for C_Bore 
									points_ScrewHead(@xcl[i],ycl,0,(@h_dia/2),dir_y) #@pthd[0]
									@pthd=@pthd.collect {|point| point.transform(backt)}
									points_Screw(@xcl[i],ycl,(@z1*-1),(@s_dia/2),dir_y) #@ptsc[0]
									@ptsc=@ptsc.collect {|point| point.transform(backt)}
									cface=codef.entities.add_face(@pthd)
									codef.entities.erase_entities cface
									cface=codef.entities.add_face(@ptsc)
									orig_ents = codef.entities.to_a
									#UI.messagebox("Orig no of enities = " + orig_l.to_s)
									cface.pushpull((@edge3L-@z1)*-1*dir_y)
									codef.entities.erase_entities cface
									fin_ents = codef.entities.to_a
									added_ents = fin_ents - orig_ents
									added_ents.each {|e| e.reverse! if e.is_a?(Sketchup::Face) }
									for k in -1...11
										cface=codef.entities.add_face(@pthd[k],@pthd[k+1],@ptsc[k+1],@ptsc[k])               
									end # for k
								when "C_Bore"
									points_ScrewHead(@xcl[i],ycl,0,(@h_dia/2),dir_y) #@pthd[0]
									@pthd=@pthd.collect {|point| point.transform(backt)}
									points_Screw(@xcl[i],ycl,(@z1*-1),(@s_dia/2),dir_y) #@ptsc[0]
									@ptsc=@ptsc.collect {|point| point.transform(backt)}
									cface=codef.entities.add_face(@pthd)
									cface.pushpull(@z1*-1)
									cface=codef.entities.add_face(@ptsc)
									cface.pushpull((@edge3L-@z1)*-1)
								when "Straight"
									points_Screw(@xcl[i],ycl,0,(@s_dia/2),dir_y) #@ptsc[0]
									@ptsc=@ptsc.collect {|point| point.transform(backt)}
									cface=codef.entities.add_face(@ptsc)
									cface.pushpull((@edge3L)*-1)
							end #case @type
									#Feature added to First component at this point
							i=i+1
							n=n-1
						end #while n > 0
						Sketchup.active_model.commit_operation
						self.reset
						#Program is Complete and ends
						#Sketchup.send_action "selectSelectionTool:" 
					end #face==nil
				else
					UI.messagebox "Selection invalid "
				end #if(@ip.edge
			else
				UI.messagebox "Selection invalid"	
			end #if(@ip.valid?)	
		end #elsif @status==
	end # def onLButtonDown
	
	def onCancel(flag, view)
		view.refresh if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
	end
	
	def onRButtonDown(flags,x,y,view)
		self.reset
		self.data_Input
		self.no_of_Offset_Input
		if @noOffsets >= 1
			input_Offsets
		end
	end

end #end class K2_Tools_ScrewHoles



 file_loaded("K2_Tools_ScrewHoles.rb")
 
